unit mt4client;

{$mode objfpc}{$H+}

interface

uses
  SysUtils,
  service_intf,
  blcksock;

// Festlegung der IP Adresse und des Ports
const
  SERVER_ADDRESS = 'localhost';
  SERVER_PORT = 5000;

// Clientverbindung zum Server herstellen
procedure InitConnection(SERVER_ADDRESS: string; SERVER_PORT: longint);
// Senden und empfangen der Daten
function TCPSend(const Bid, Ask: TDataType): TDataType;
// Client vom Server trennen
procedure CloseConnection();

implementation

var
  SocketCnx: TTCPBlockSocket = nil;

procedure InitConnection(SERVER_ADDRESS: string; SERVER_PORT: longint);
var
  cnx: TTCPBlockSocket;
begin
  if (SocketCnx <> nil) then
    CloseConnection();
  if (SocketCnx = nil) then
  begin
    cnx := TTCPBlockSocket.Create();
    try
      cnx.RaiseExcept := True;
      cnx.Connect(SERVER_ADDRESS, IntToStr(SERVER_PORT));
      SocketCnx := cnx;
    except
      on e: Exception do
      begin
        cnx.Free();
        FreeAndNil(SocketCnx);
        raise;
      end;
    end;
  end;
end;

procedure CloseConnection();
begin
  if (SocketCnx <> nil) then
  begin
    FreeAndNil(SocketCnx);
  end;
end;

function TCPSend(const Bid, Ask: TDataType): TDataType;
var
  RequestBuffer: TMT4Request;
  ResponseBuffer: TMT4Response;
  cnx: TTCPBlockSocket;
  bufferLen: integer;
begin
  RequestBuffer.Bid := Bid;
  RequestBuffer.Ask := Ask;
  if (SocketCnx <> nil) then
  begin
    cnx := SocketCnx;
    cnx.ExceptCheck();
    cnx.SendBuffer(@RequestBuffer, SizeOf(RequestBuffer));
    cnx.ExceptCheck();
    bufferLen := cnx.RecvBufferEx(@ResponseBuffer, SizeOf(ResponseBuffer), 200);
    cnx.ExceptCheck();
    if (bufferLen <> SizeOf(ResponseBuffer)) then
      raise Exception.Create('Invalid server response.');
    Result := ResponseBuffer.ResultArg;
  end
  else
    Result := -1;
end;

end.

