unit MainForm;

{$mode objfpc}{$H+}

interface

uses
  server_imp,
  service_intf,
  SysUtils,
  Forms,
  StdCtrls,
  ComCtrls,
  Menus,
  TAGraph,
  TAMultiSeries,
  Math;

type

  { TForm1 }
  TForm1 = class(TForm)
    Chart: TChart;
    BoxAndWhisker: TBoxAndWhiskerSeries;
    TCAufloesung01: TMenuItem;
    TBAufloesung: TEdit;
    TBLabelTick1: TLabel;
    TCAufloesung: TMenuItem;
    PageControl1: TPageControl;
    ToolButton3: TToolButton;
    ToolButton4: TToolButton;
    TSChart: TTabSheet;
    TBEditTick: TEdit;
    TBLabelTick: TLabel;
    MMclear: TMenuItem;
    MMShow: TMenuItem;
    TBEditAskValue: TEdit;
    TBEditBidValue: TEdit;
    TBLabelAsk: TLabel;
    TBLabelBid: TLabel;
    MainMenu: TMainMenu;
    Memo1: TMemo;
    MMemo: TMenuItem;
    MServerStopp: TMenuItem;
    ToolBar1: TToolBar;
    ToolButton1: TToolButton;
    ToolButton2: TToolButton;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure MMShowClick(Sender: TObject);
    procedure MMclearClick(Sender: TObject);
    procedure MServerStoppClick(Sender: TObject);
    procedure TBEditTickChange(Sender: TObject);
  private
    { private declarations }
    procedure compute;
    function CalculateBars(const Aufloesung: longint): TMT4TickBar;
  public
    { public declarations }
    TInput: TMT4Request;
    TOutput: TMT4Response;
    THistory: TMT4History;
    THistoryTickBar: TMT4TickBar;
    THistoryTickChart: TMT4HistoryTickChart;
    Zaeler: longint;
  end;

var
  Form1: TForm1;
  serverThread: TServerListnerThread;


implementation

{$R *.lfm}

{ TForm1 }

procedure TForm1.FormCreate(Sender: TObject);
begin
  try
    begin
      serverThread := TServerListnerThread.Create('localhost', 5000);
      Memo1.Append('Server listen on localhost, Port: 5000');
    end;
  except
    Memo1.append('Server nicht gestartet');
  end;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  serverThread.Terminate();
  Sleep(500);
  serverThread.Free();
  Application.Terminate;
end;

procedure TForm1.MMShowClick(Sender: TObject);
begin
  if MMShow.Caption = 'Protokoll zeigen' then
  begin
    MMShow.Caption := 'Protokoll verstecken';
    Form1.Memo1.Visible := True;
  end
  else
  begin
    MMShow.Caption := 'Protokoll zeigen';
    Form1.Memo1.Visible := False;
  end;
end;

procedure TForm1.MMclearClick(Sender: TObject);
begin
  Form1.Memo1.Clear;
end;

procedure TForm1.MServerStoppClick(Sender: TObject);
begin
  Application.Terminate;
end;

procedure TForm1.TBEditTickChange(Sender: TObject);
var
  ylist: array [1..4] of double;
  y: double;
  i: integer;
  Aufloesung: longint;

  Count: integer;
  Candel: integer;
  Bar: integer;

begin
  Aufloesung := StrToInt(TBAufloesung.Text);
  compute();
  // Augabe des Ask-Kurses und des Zälers
  TBEditBidValue.Text := FloatToStr(TOutput.Bid);
  TBEditAskValue.Text := FloatToStr(TOutput.Ask);
  // Einzeltick an THistory übergeben, THistory muß dabei jeweils um
  // einen Speicherplatz erweitert werden.
  SetLength(THistory, Zaeler);
  // Den letzten THistoryplatz mit TOutput(später TInput) füllen
  THistory[Zaeler - 1] := TOutput;
  FloatToStr(THistory[Zaeler - 1].bid);
  SetLength(THistoryTickChart, Length(THistory));
  Bar := length(THistory) div Aufloesung;
  THistoryTickChart[Bar] := CalculateBars(Aufloesung);
  y := THistoryTickChart[Bar].LOMCH[0];
  for i := 1 to 4 do     // Open, Median, Close, High
  begin
    ylist[i] := THistoryTickChart[Bar].LOMCH[i];
  end;
  BoxAndWhisker.ListSource.YCount := 5;
  Count := BoxAndWhisker.Count;
  if (Count > 0) and (Count > Bar) then
    BoxAndWhisker.Delete(Count - 1);
  Candel := BoxAndWhisker.AddXY(Bar, y);
  BoxAndWhisker.ListSource.SetYList(Candel, ylist);
end;

function TForm1.CalculateBars(const Aufloesung: longint): TMT4TickBar;
var
  TempHistory: array of TDataType;
  k, i: longint;
  PLow, POpen, PMean, PClose, PHigh: TDataType;
begin
  // und das temporäre Array mit der Größe des letzten unvollständigen
  // Auflösungsblocks definieren.
  k := (Zaeler - 1) mod Aufloesung;
  POpen := THistory[High(THistory) - k].Bid;
  PClose := THistory[High(THistory)].Bid;
  SetLength(TempHistory, k + 1);
  i := -1;
  repeat
    i += 1;
    TempHistory[i] := THistory[High(THistory) - i].Bid;
  until i = k;
  if k > 0 then
  begin
    PMean := Mean(TempHistory);
    PLow := MinValue(TempHistory);
    PHigh := MaxValue(TempHistory);
  end
  else
  begin
    PMean := POpen;
    PLow := POpen;
    PHigh := POpen;
  end;

  if POpen < PClose then
  begin
    THistoryTickBar.LOMCH[1] := POpen;
    THistoryTickBar.LOMCH[3] := PClose;
  end
  else
  begin
    THistoryTickBar.LOMCH[3] := POpen;
    THistoryTickBar.LOMCH[1] := PClose;
  end;

  THistoryTickBar.LOMCH[0] := PLow;
  THistoryTickBar.LOMCH[2] := PMean;
  THistoryTickBar.LOMCH[4] := PHigh;
  Result := THistoryTickBar;
end;

procedure TForm1.compute;
begin
  TOutput.Bid := TInput.Bid;
  TOutput.Ask := TInput.Ask;
  TOutput.ResultArg := TInput.Bid - TInput.Ask;
end;

end.

