unit server_imp;

{$mode objfpc}{$H+}

interface

uses
  Classes,
  SysUtils,
  blcksock,
  synsock,
  service_intf;

type

  { TServerListnerThread }
  {______________________________________________________________________________}
  TServerListnerThread = class(TThread)
  private
    FDefaultTimeOut: integer;
    FServerSocketObject: TTCPBlockSocket;
    FIP: string;
    FPort: integer;
  public
    FZaeler: longint;
    constructor Create(const AIP: string; const APort: integer);
    destructor Destroy(); override;
    procedure Execute(); override;
    property DefaultTimeOut: integer read FDefaultTimeOut write FDefaultTimeOut;
  end;

  { TClientHandlerThread }
  {______________________________________________________________________________}
  TClientHandlerThread = class(TThread)
  private
    FDefaultTimeOut: integer;
    FSocketObject: TTCPBlockSocket;
    FSocketHandle: TSocket;
  private
    FInput: TMT4Request;
    FOutput: TMT4Response;
    procedure ClearBuffers();
    function ReadInputBuffer(): boolean;
    procedure SendOutputBuffer();
  protected
    procedure SyncInput;
  public
    constructor Create(ASocketHandle: TSocket);
    procedure Execute(); override;
    property DefaultTimeOut: integer read FDefaultTimeOut write FDefaultTimeOut;
  end;

implementation

uses
  MainForm;

var
  Zaeler: longint;

{ TServerListnerThread }
{______________________________________________________________________________}
constructor TServerListnerThread.Create(const AIP: string; const APort: integer);
begin
  FServerSocketObject := TTCPBlockSocket.Create();
  FreeOnTerminate := False;
  FDefaultTimeOut := 1000;
  FIP := AIP;
  FPort := APort;
  inherited Create(False);
end;

destructor TServerListnerThread.Destroy();
begin
  FreeAndNil(FServerSocketObject);
  inherited Destroy();
end;

procedure TServerListnerThread.Execute();
var
  ClientSock: TSocket;
begin
  FZaeler := 0;
  with FServerSocketObject do
  begin
  RaiseExcept := True;
  CreateSocket();
  SetLinger(True, 10);
  Bind(FIP, IntToStr(FPort));
  Listen();
  end;
  while not Terminated do
  begin
    if FServerSocketObject.CanRead(DefaultTimeOut) then
    begin
      ClientSock := FServerSocketObject.Accept();
      TClientHandlerThread.Create(ClientSock);
    end;
  end;
end;

{ TClientHandlerThread }
{______________________________________________________________________________}
procedure TClientHandlerThread.SyncInput;
begin
  Form1.TInput := FInput;
  Form1.Zaeler := Zaeler;
  FOutput := Form1.TOutput;
  Form1.TBEditTick.Text := IntToStr(Zaeler);
end;

procedure TClientHandlerThread.ClearBuffers();
begin
  FillChar(FInput, SizeOf(FInput), #0);
  FillChar(FOutput, SizeOf(FOutput), #0);
end;

function TClientHandlerThread.ReadInputBuffer(): boolean;
var
  strBuff: string;
  expectedBufferLen: longint;
  readBufferLen: PtrInt;
begin
  Result := False;
  expectedBufferLen := SizeOf(FInput);
  SetLength(strBuff, expectedBufferLen);
  readBufferLen := FSocketObject.RecvBufferEx(@(strBuff[1]),
    expectedBufferLen, DefaultTimeOut);
  if (readBufferLen = 0) and (FSocketObject.LastError = WSAETIMEDOUT) then
    Result := False
  else
  begin
    FSocketObject.RaiseExcept := True;
    FSocketObject.ExceptCheck();
    FSocketObject.RaiseExcept := False;
    if (readBufferLen <> expectedBufferLen) then
      raise Exception.Create('Invalid stream.');
    Move(strBuff[1], FInput, expectedBufferLen);
    Result := True;
  end;
end;

procedure TClientHandlerThread.SendOutputBuffer();
begin
  FSocketObject.SendBuffer(@FOutput, SizeOf(FOutput));
end;

constructor TClientHandlerThread.Create(ASocketHandle: TSocket);
begin
  FSocketHandle := ASocketHandle;
  FreeOnTerminate := True;
  FDefaultTimeOut := 200;
  inherited Create(False);
end;

procedure TClientHandlerThread.Execute();
begin
  FSocketObject := TTCPBlockSocket.Create();
  try
    FSocketObject.Socket := FSocketHandle;
    FSocketObject.GetSins();
    while not Terminated do
    begin
      ClearBuffers();   // Alles auf #0 setzen
      if ReadInputBuffer() then
      begin
        Zaeler := Zaeler + 1;
        Synchronize(@SyncInput);
        SendOutputBuffer();
        ClearBuffers();
      end
      else
        FreeAndNil(FSocketObject);
    end;
  finally
    FreeAndNil(FSocketObject);
  end;
end;

end.

